const React = require('react');

const PropTypes = React.PropTypes;

const PINK = 'pink';
const BLUE = 'blue';
const YELLOW = 'yellow';
const GREEN = 'green';
const COLORS = {};
COLORS[PINK] = '#FFCCCC';
COLORS[BLUE] = '#CCFFFF';
COLORS[YELLOW] = '#FFFFCC';
COLORS[GREEN] = '#CCFFCC';
const TEXT_MAX_DEFAULT = 287; // text longer than this will generally overflow
const TEXT_MAX_LARGE = 558; // when 'large' is true
const RAND_TILT_MAX = 10;

const StickyNote = React.createClass({
  propTypes: {
    children: PropTypes.node.isRequired,
    large: PropTypes.bool,
    color: PropTypes.oneOf([PINK, BLUE, YELLOW, GREEN]),
    customTextLimit: PropTypes.number,
    tiltInDegrees: PropTypes.number, // will be randomized if not provided
  },

  componentWillMount: function() {
    let textMax = this._calculateTextMax();
    if (this._calculateChildrenLength() > textMax) {
      console.warn(`WARNING: Please limit length of content of <StickyNote> to \
${textMax} or it will overflow!`);
    }
  },

  /**
   * @return {Object}
   */
  getDefaultProps: function() {
    return {
      color: YELLOW,
    };
  },

  /**
   * @return {Object}
   */
  getInitialState: function() {
    return {
      highlighted: false,
    };
  },

  _color: function() {
    return COLORS[this.props.color];
  },

  _calculateTextMax: function() {
    return this.props.customTextLimit ||
      (this.props.large ? TEXT_MAX_LARGE : TEXT_MAX_DEFAULT);
  },

  _calculateChildrenLength: function() {
    let lengthCounter = 0;
    const countLengthOfChild = function countLengthOfChild(child) {
      let textContent;
      if (typeof child === 'string') {
        textContent = child;
      } else {
        if (child.props && child.props.children) {
          React.Children.forEach(child.props.children, countLengthOfChild);
        }
      }
      if (textContent) {
        lengthCounter += textContent.length;
      }
    };
    React.Children.forEach(this.props.children, countLengthOfChild);
    return lengthCounter;
  },

  _randomTilt: function() {
    return Math.floor(Math.random() * RAND_TILT_MAX - 5)
  },

  render: function() {
    const size = this.props.large ? '300px' : '200px';
    let tilt = this.props.tiltInDegrees;
    if (typeof this.props.tiltInDegrees === 'undefined') {
      tilt = this._randomTilt();
    }

    const styles = {
      fontFamily: '"Courier New",Courier,"Lucida Sans Typewriter","Lucida Typewriter",monospace',
      width: size,
      height: size,
      padding: '1.5em',
      backgroundColor: this._color(),
      overflowY: 'scroll',
      // tilt
      webkitTransform: `rotate(${tilt}deg)`,  // Chrome, Opera 15+, Safari 3.1+
      msTransform: `rotate(${tilt}deg)`,  // IE 9
      transform: `rotate(${tilt}deg)`,  // Firefox 16+, IE 10+, Opera
    };

    return (
      <div
        className='sticky-note'
        style={styles}
      >
      {this.props.children}
      </div>
    );
  }
});

module.exports = StickyNote;
